--
-- This file is part of TALER
-- Copyright (C) 2014--2022 Taler Systems SA
--
-- TALER is free software; you can redistribute it and/or modify it under the
-- terms of the GNU General Public License as published by the Free Software
-- Foundation; either version 3, or (at your option) any later version.
--
-- TALER is distributed in the hope that it will be useful, but WITHOUT ANY
-- WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-- A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along with
-- TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
--

CREATE TABLE IF NOT EXISTS auditor_pending_deposits
(
    row_id BIGINT GENERATED BY DEFAULT AS IDENTITY UNIQUE PRIMARY KEY,
    total_amount taler_amount NOT NULL,
    wire_target_h_payto BYTEA NOT NULL CHECK (LENGTH(wire_target_h_payto)=32),
    batch_deposit_serial_id BIGINT NOT NULL,
    deadline BIGINT NOT NULL,
    suppressed BOOLEAN NOT NULL DEFAULT FALSE
);

COMMENT ON TABLE auditor_pending_deposits
  IS 'Reports when deposits are delayed (can be harmless, if the aggregation and wire transfer are made shortly afterwards)';

COMMENT ON COLUMN auditor_pending_deposits.row_id
  IS 'Unique identifier of the report in the auditor database';
COMMENT ON COLUMN auditor_pending_deposits.batch_deposit_serial_id
  IS 'Serial ID of the deposit operation in the exchange database';
COMMENT ON COLUMN auditor_pending_deposits.wire_target_h_payto
  IS 'Hash of the bank account into which the deposit should have been made';
COMMENT ON COLUMN auditor_pending_deposits.total_amount
  IS 'Total transaction amount impacted';
COMMENT ON COLUMN auditor_pending_deposits.suppressed
  IS 'True if the report was suppressed by an administrator';
