/*
  This file is part of TALER
  Copyright (C) 2022 Taler Systems SA

  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU Affero General Public License as published by the Free Software
  Foundation; either version 3, or (at your option) any later version.

  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License along with
  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
*/
/**
 * @file taler-exchange-httpd_contract.h
 * @brief Handle /coins/$COIN_PUB/contract requests
 * @author Florian Dold
 * @author Benedikt Mueller
 * @author Christian Grothoff
 */
#ifndef TALER_EXCHANGE_HTTPD_CONTRACT_H
#define TALER_EXCHANGE_HTTPD_CONTRACT_H

#include <gnunet/gnunet_util_lib.h>
#include <microhttpd.h>
#include "taler-exchange-httpd.h"


/**
 * Handle a GET "/contracts/$C_PUB" request. Returns the
 * encrypted contract.
 *
 * @param rc request context
 * @param args array of additional options (length: 1, first is the contract_pub)
 * @return MHD result code
  */
MHD_RESULT
TEH_handler_contracts_get (struct TEH_RequestContext *rc,
                           const char *const args[1]);


#endif
